package velha;

/**
 * <p>Title: Tabuleiro</p>
 * <p>Description: Classe que descreve um tabuleiro de um jogo da velha.
 *  implementada com um array de 9 elementos e mtodos de marcar clula e verificar
 * se o jogo foi ganho.</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: CIn - UFPE</p>
 * @author Everson Melo (elsm2)
 * @author Lamberto Augusto (laon)
 * @author Milena Andrade (maag)
 * @author Pablo Barbosa (pab)
 * @version 1.0
 */

public class Tabuleiro {

  private int celula[] = new int[9];
  private int jogadas = 0;

  public Tabuleiro() {
  }

  /**
   * Mtodo que marca uma clula do jogo da velha com bola ou cruz.
   * @param posicao A posio da jogada.
   * @param valor Se ser marcada uma bola ou uma cruz.
   * @return Um booleano indicando se o jogo foi ganho aps a jogada.
   * @throws CelulaCheiaException Se a clula ja tiver sido utilizada anteriormente.
   */
  public boolean marcarCelula(int posicao, int valor) throws
      CelulaCheiaException {
    if (celula[posicao] == 0) {
      celula[posicao] = valor;
      if (checarLinha(posicao) || checarColuna(posicao) ||
          checarDiagonal(posicao)) {
        return true;
      }
      else {
        return false;
      }
    }
    else {
      throw new CelulaCheiaException();
    }
  }

  private boolean checarLinha(int posicao) {
    boolean venceu = false;
//checa a primeira linha
    if (posicao >= 0 && posicao <= 2) {
      if (celula[0] == celula[1] && celula[0] == celula[2]) {
        venceu = true;
      }
    }
    else {
//checa a segunda linha
      if (posicao >= 3 && posicao <= 5) {
        if (celula[3] == celula[4] && celula[3] == celula[5]) {
          venceu = true;
        }
      }
      else {
//checa a terceira linha
        if (celula[6] == celula[7] && celula[6] == celula[8]) {
          venceu = true;
        }
      }
    }
    return venceu;
  }

  private boolean checarColuna(int posicao) {
    boolean venceu = false;
// checa a primeira coluna
    if (posicao % 3 == 0) {
      if (celula[0] == celula[3] && celula[0] == celula[6]) {
        venceu = true;
      }
    }
    else {
//checa a segunda coluna
      if (posicao % 3 == 1) {
        if (celula[1] == celula[4] && celula[1] == celula[7]) {
          venceu = true;
        }
      }
      else {
//checa a terceira coluna
        if (celula[2] == celula[5] && celula[2] == celula[8]) {
          venceu = true;
        }
      }
    }
    return venceu;
  }

  private boolean checarDiagonal(int posicao) {
    boolean venceu = false;
//checa diagonal crescente
    if (posicao == 2 || posicao == 6) {
      if (celula[2] == celula[4] && celula[2] == celula[6]) {
        venceu = true;
      }
    }
    else {
//checa diagonal decrescente
      if (celula[0] == celula[4] && celula[0] == celula[8]) {
        venceu = true;
      }
    }
    return venceu;
  }

  public boolean empatou() {
    if (jogadas == 9) {
      return true;
    }
    else {
      return false;
    }
  }

  public int getJogadas() {
    return this.jogadas;
  }

  public boolean acabou() {
    boolean empate = false;
    boolean naoAcabou = false;
    boolean fim = false;

    if (this.checarLinha(0) || this.checarLinha(1) || this.checarLinha(2) ||
        this.checarColuna(0) || this.checarColuna(1) || this.checarColuna(2) ||
        this.checarDiagonal(0) || this.checarDiagonal(2)) {
      fim = true;
    }
    else {
      if (this.empatou()) {
        empate = true;
      }
    }
    if (empate || fim) {
      fim = true;
    }
    else {
      fim = false;
    }
    return fim;
  }

  public void incremetarJogada() {
    jogadas++;
  }

  public boolean XVenceu() {
    boolean retorno = false;

    if (this.acabou()) {
      if (!this.empatou()) {

        //Verificando linhas
        //linha 0
        if (this.celula[0] == 1 && this.celula[1] == 1 &&
            this.celula[2] == 1) {
          retorno = true;
        }
        else {
          //Linha 1
          if (this.celula[3] == 1 && this.celula[4] == 1 &&
              this.celula[5] == 1) {
            retorno = true;
          }
          else {
            //Linha 2
            if (this.celula[6] == 1 && this.celula[7] == 1 &&
                this.celula[8] == 1) {
              retorno = true;
            }
            else {
              //Verificando colunas
              //Coluna 1
              if (this.celula[0] == 1 && this.celula[3] == 1 &&
                  this.celula[6] == 1) {
                retorno = true;
              }
              else {
                //Coluna 2
                if (this.celula[1] == 1 && this.celula[4] == 1 &&
                    this.celula[7] == 1) {
                  retorno = true;
                }
                else {
                  //Coluna 3
                  if (this.celula[3] == 1 &&
                      this.celula[5] == 1 &&
                      this.celula[8] == 1) {
                    retorno = true;
                  }
                  else {
                    //Diagonal 1
                    if (this.celula[0] == 1 && this.celula[4] == 1 &&
                        this.celula[8] == 1) {
                      retorno = true;
                    }
                    else {
                      //Diagonal 2
                      if (this.celula[2] == 1 &&
                          this.celula[4] == 1 &&
                          this.celula[6] == 1) {
                        retorno = true;
                      }
                      else {
                        retorno = false;
                      }
                    } //Vendo diagonal 2
                  } //Diagonal 1 e 2
                } //Coluna 3
              }
            }
          }

        }
      } //Houve um empate
    } //O jogo nao acabou
    return retorno;
  }

}
