package velha;

import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import java.util.Vector;
import java.util.Iterator;
import sun.rmi.registry.RegistryImpl;

/**
 * Universidade Federal de Pernambuco
 * Centro de Informtica
 * Disciplina: Infra-Estrutura de Software
 * Projeto: Jogo da Velha
 *
 * Tipo:     VelhaServerImpl
 * Alunos: Everson Luis da Silva Melo (elsm2@cin.ufpe.br)
 *         Lamberto Augusto de Oliveira Neto (laon@cin.ufpe.br)
 *         Millena de Andrade Almeida Gomes (maag@cin.ufpe.br)
 *         Pablo Arajo Borges (pab@cin.ufpe.br)
 */

 public class VelhaServerImpl extends UnicastRemoteObject
                              implements VelhaServer, Cloneable , java.io.Serializable{

      private Vector players;
      private Vector convites;

      public VelhaServerImpl() throws RemoteException{
          super();
          players = new Vector();
          convites = new Vector();
      }

      public String getClient() throws RemoteException{
          String retorno = null;
          Players ultimo = (Players)players.lastElement();
          retorno = ultimo.getNick();
          return retorno;
      }

      public boolean hasClient() throws RemoteException{
          boolean retorno = true;
          if(players.isEmpty()){
             retorno = false;
          }
          return retorno;
      }

      /**
       * Mtodo que insere um cliente na lista de clientes on-line. Este mtodo
       * deve ser chamado sempre que um cliente se conectar.
       */

      public void insertion(Players novo) throws RemoteException{
        if(novo != null){
          if(!players.contains(novo)){
            players.addElement(novo);
          }
        }
      }

      public boolean convidarJog(FramePrincipal p, String convidado) throws RemoteException{
        boolean retorno = false;
        Iterator it = players.iterator();
        int indice = 0;
        Players prox = new Players();
        while(it.hasNext()){
           prox = (Players)it.next();
           if(prox.getNick().equals(convidado)){
             indice = players.indexOf(prox);
           }
        }
        if(convites.get(indice) != null){
            retorno = true;
        }
        else{
          convites.add(indice, p);
        }
        return retorno;
      }

      public FramePrincipal temConvite(String nick) throws RemoteException{
         FramePrincipal retorno = null;
         Iterator it = players.iterator();
         int indice = 0;
         Players prox = new Players();
         while(it.hasNext()){
           prox = (Players)it.next();
           if(prox.getNick().equals(nick)){
             indice = players.indexOf(prox);
           }
         }
         if(convites.get(indice) != null){
            retorno = (FramePrincipal)convites.get(indice);
         }
         return retorno;
      }

      /**
       * Metodo que o servidor fica verificando se os clientes on-line da lista
       * sao validos.
       */

      public void update() throws RemoteException, MalformedURLException{
        String[] ips = Naming.list("localhost");
        boolean status = false;
        Iterator it = players.iterator();
        Players prox = null;
        while(it.hasNext() && !status){
          prox = (Players)it.next();
          for(int i = 0; i < ips.length; i++){
            if(prox.getServidor().equals(ips[i])){
              status = true;
            }
          }
          if(!status){
            players.removeElement(prox);
          }
          else{
            status = false;
          }
        }
      }

      public void remove(Jogador jog) throws RemoteException{
        if(jog != null){
          Iterator it = players.iterator();
          Players prox = new Players();
          while(it.hasNext()){
             prox = (Players)it.next();
             if(prox.getNick().equals(jog.getNome())){
               players.removeElement(prox);
             }
          }
        }
        else{
          System.err.println("No foi possivel remover o jogador!");
        }
      }

      public void remove(String jog) throws RemoteException{
        if(jog != null){
          Iterator it = players.iterator();
          Players prox = new Players();
          while(it.hasNext()){
             prox = (Players)it.next();
             if(prox.getNick().equals(jog)){
               players.removeElement(prox);
             }
          }
        }
        else{
          System.err.println("No foi possivel remover o jogador!");
        }
      }

      /**
       * Metodo que retorna todos os clientes da lista on-line
       */

      public Vector getAllClients() throws RemoteException{
          return players;
      }

      public static void main(String[] args){
        try{
          System.err.println("Inicializando servidor: Por Favor Aguarde.");
          VelhaServerImpl servidor = new VelhaServerImpl();
          RegistryImpl r = new RegistryImpl(1099);
          Naming.rebind("//localhost/VelhaServer", servidor);

          //ThreadUpdate thread = new ThreadUpdate(servidor);
          //thread.start();

          //ThreadEnviar envia = new ThreadEnviar(servidor);
          //envia.start();

          System.err.println("O servidor esta rodando!!!");
        }
        catch(Exception e){
          e.printStackTrace();
        }
      }
 }
